@@ -460,7 +460,17 @@ class Grd(ABC):
     @figure
     def tricontourf(self, axes=None, show=True, figsize=None, cbar=False, **kwargs):
         if len(self.triangles) > 0:
-            ax = axes.tricontourf(self.x, self.y, self.triangles, self.values, **kwargs)
+            values = self.values.values  # this is a numpy array
+            if values.shape[1] > 1:
+                values = np.linalg.norm(values, ord=2, axis=1)
+            get_idx = self.nodes.index.get_loc
+            ax = axes.tricontourf(
+                self.x,
+                self.y,
+                [[get_idx(n_id) for n_id in tri] for tri in self.triangles.values],
+                values.squeeze(),
+                **kwargs,
+            )
             if cbar is True:
                 plt.colorbar(ax)
         return axes
@@ -468,7 +478,17 @@ class Grd(ABC):
     @figure
     def tripcolor(self, axes=None, show=True, figsize=None, **kwargs):
         if len(self.triangles) > 0:
-            axes.tripcolor(self.x, self.y, self.triangles, self.values, **kwargs)
+            values = self.values.values  # this is a numpy array
+            if values.shape[1] > 1:
+                values = np.linalg.norm(values, ord=2, axis=1)
+            get_idx = self.nodes.index.get_loc
+            axes.tripcolor(
+                self.x,
+                self.y,
+                [[get_idx(n_id) for n_id in tri] for tri in self.triangles.values],
+                values.squeeze(),
+                **kwargs,
+            )
         return axes
 
     @figure
@@ -478,7 +498,13 @@ class Grd(ABC):
         if len(self.triangles) > 0:
             kwargs.update({'linewidth': linewidth})
             kwargs.update({'color': color})
-            axes.triplot(self.x, self.y, self.triangles, **kwargs)
+            get_idx = self.nodes.index.get_loc
+            axes.triplot(
+                self.x,
+                self.y,
+                [[get_idx(n_id) for n_id in tri] for tri in self.triangles.values],
+                **kwargs,
+            )
         return axes
 
     @figure
